<?php
/**
 * Core Design Accordion plugin for Joomla! 1.5
 * @author      Daniel Rataj, <info@greatjoomla.com>
 * @package     Joomla 
 * @subpackage	Content
 * @category    Plugin 
 * @version     1.0.3
 * @copyright   Copyright (C)  2007 - 2008 Core Design, http://www.greatjoomla.com
 * @license		http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import library dependencies
jimport('joomla.plugin.plugin');

class plgContentCdAccordion extends JPlugin
{

    /**
     * Constructor
     *
     * For php4 compatability we must not use the __constructor as a constructor for plugins
     * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
     * This causes problems with cross-referencing necessary for the observer design pattern.
     *
     * @param object $subject The object to observe
     * @param object $params  The object that holds the plugin parameters
     * @since 1.5
     */
    function plgContentCdAccordion(&$subject)
    {
        parent::__construct($subject);

        // load plugin parameters
        $this->plugin = &JPluginHelper::getPlugin('content', 'cdaccordion');
        $this->params = new JParameter($this->plugin->params);
    }

    /**
     *
     * Method is called by the view
     *
     * @param 	object		The article object.  Note $article->text is also available
     * @param 	object		The article params
     * @param 	int			The 'page' number
     */
    function onPrepareContent(&$article, &$params, $limitstart = 0)
    {
        // define the regular expression for the bot
        $regex = "#{accordion(?:\s?(.*?)?)?}(.*?){/accordion}#is";

        if (!preg_match($regex, $article->text))
        {
            return;
        }
        
        // define language
        JPlugin::loadLanguage('plg_content_cdaccordion', JPATH_ADMINISTRATOR);

        if (!defined('_JSCRIPTEGRATOR'))
        {
            JError::raiseNotice('', JText::_('CDACC_ENABLE_SCRIPTEGRATOR'));
            return;
        }
        
        // require Scriptegrator version 1.3.5 or higher
        $version = '1.3.5';
        if (!JScriptegrator::versionRequire($version))
        {
            JError::raiseNotice('', JText::sprintf('CDACC_SCRIPTEGRATOR_REQUIREVERSION', $version));
            return;
        }

        if (!JScriptegrator::checkLibrary('jquery', 'site'))
        {
            JError::raiseNotice('', JText::_('CDACC_MISSING_JQUERY'));
            return;
        }

        // Explication:
        // $match[1]	-> parameters
        // $match[2]	-> text to tabs

        // load database parameters
        $compress = (int)$this->params->get('compress', 1);
        // end

        // insert JS scripts and CSS style to the head
        $this->loadHeadScripts($compress);

        // replacement regular expression
        $article->text = preg_replace_callback($regex, array($this, 'replacer'), $article->
            text);
    }

    /**
     * Load head scripts and CSS styles
     *
     * Function to load head scripts and CSS styles
     *
     * @access	public
     * @param	string $string
     * @return	void
     */

    function loadHeadScripts($compress = 1)
    {
        $document = &JFactory::getDocument(); // set document for next usage
        $live_path = JURI::base(true) . '/'; // define live site

        JScriptegrator::UILoader($compress, 'ui.accordion'); // load jQuery UI Accordion

        $document->addScript($live_path .
            'plugins/content/cdaccordion/utils/js/jquery/jsloader.php?compress=' . (string )
            $compress . '');
    }
    // end --------------------------------------------------------------------


    /**
     * Create a Random String
     *
     * Useful for generating passwords or hashes.
     *
     * @access	public
     * @param	string 	type of random string.  Options: alunum, numeric, nozero, unique
     * @param	integer	number of characters
     * @return	string
     */
    function randomString($length = 5)
    {
        $alphanum = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        $var_random = '';
        mt_srand(10000000 * (double)microtime());
        for ($i = 0; $i < (int)$length; $i++)
            $var_random .= $alphanum[mt_rand(0, 61)];
        unset($alphanum);
        return $var_random;
    }

    /**
     * Plugin replacer
     */

    function replacer(&$match)
    {
        $document = &JFactory::getDocument();

        $live_path = JURI::base(true) . '/'; // define live site

        $compress = (int)$this->params->get('compress', 1);

        $default_mode = $this->params->get('default_mode', 'manually');
        $default_style = $this->params->get('default_style', 'smoothness');
        if ($default_style == -1)
        {
            $default_style = 'smoothness';
        }
        
        $default_box = $this->params->get('default_box', '');
        $default_select = (int)$this->params->get('default_select', 0);
        $default_event = $this->params->get('default_event', 'click');
        $default_animated = $this->params->get('default_animated', 'slide');
        $default_autoheight = (int)$this->params->get('default_autoheight', 0);
        $default_spacebetween = $this->params->get('default_spacebetween', '2px');
        $readmore = (int)$this->params->get('readmore', 0);
        $process_plugins = (int)$this->params->get('plugins', 0);


        if ($match[1])
        {
            $settings = $match[1];
        } else
        {
            $settings = '';
        }

        if ($match[2])
        {
            $set_articles = $match[2];
        } else
        {
            return;
        }
        unset($match);
        
        // tab mode - source, article
        if (preg_match('#mode\s?=#', $settings))
        {
            preg_match('#mode\s?=\s?(id|manually)#', $settings, $mode);
            $mode = $mode[1];
        } else
        {
            $mode = $default_mode;
        }

        if (!$mode)
        {
            JError::raiseNotice('', JText::_('CDACC_NO_MODE'));
            return;
        }

        // tab id
        if (preg_match('#id\s?=#', $settings))
        {
            preg_match('#id\s?=\s?([0-9a-zA-Z]{1,})#', $settings, $acc_id);
            $acc_id = $acc_id[1];
        } else
        {
            $acc_id = $this->randomString(5); // define ID for a tab
        }
        // end

        // define variables for tabs

        // style
        if (preg_match('#style\s?=#', $settings))
        {
			$files = implode('|', JScriptegrator::themeList());
			
			preg_match('#style\s?=\s?(' . $files . ')?#', $settings, $style);
			$style = ( isset($style[1]) ? $style[1] : 'smoothness');
			
			unset($files);
        } else
        {
            $style = $default_style;
        }
        // end
        
        $document->addStyleSheet($live_path . 'plugins/content/cdaccordion/css/cdaccordion.css', 'text/css');
        
    	if ($style) {
			JScriptegrator::UICssLoader($compress, $style, 'ui.core');
			JScriptegrator::UICssLoader($compress, $style, 'ui.theme');
			JScriptegrator::UICssLoader($compress, $style, 'ui.accordion');
		}
		
        // border
        if (preg_match('#box\s?=#', $settings))
        {
            preg_match('#box\s?=\s?([a-zA-Z0-9empx%.\#;:\(\) -]{1,})#', $settings, $box);
            $box = $box[1];
        } else
        {
            $box = $default_box;
            unset($default_box);
        }
        // end

        // tab select
        if (preg_match('#select\s?=#', $settings))
        {
            preg_match('#select\s?=\s?([0-9]{1,})#', $settings, $select);
            $select = $select[1];
        } else
        {
            $select = $default_select;
            unset($default_select);
        }
        
        if ($select) {
        	$select = --$select;
        } else {
        	$select = 'false';
        }
        // end

        // event
        if (preg_match('#event\s?=#', $settings))
        {
            preg_match('#event\s?=\s?(click|mouseover)#', $settings, $event);
            $event = $event[1];
        } else
        {
            $event = $default_event;
            unset($default_event);
        }
        // end

        // animated
        if (preg_match('#animated\s?=#', $settings))
        {
            preg_match('#animated\s?=\s?(slide|none|bounceslide)#', $settings, $animated);
            $animated = $animated[1];
        } else
        {
            $animated = $default_animated;
            unset($default_animated);
        }
        if ($animated == 'none')
        {
            $animated = 'false';
        } else
        {
            $animated = '\'' . $animated . '\'';
        }
        // end

        // autoheight
        if (preg_match('#autoheight\s?=#', $settings))
        {
            preg_match('#autoheight\s?=\s?(true|false)#', $settings, $autoheight);
            $autoheight = $autoheight[1];
        } else
        {
            if ($default_autoheight)
            {
                $default_autoheight = 'true';
            } else
            {
                $default_autoheight = 'false';
            }

            $autoheight = $default_autoheight;
            unset($default_autoheight);
        }
        // end

        // tab space between
        if (preg_match('#spacebetween\s?=#', $settings))
        {
            preg_match('#spacebetween\s?=\s?([0-9empx%.]{1,})#', $settings, $spacebetween);

            $spacebetween = $spacebetween[1];

        } else
        {
            $spacebetween = $default_spacebetween;
            unset($default_spacebetween);
        }
        // end

        $custom_js = "<script type=\"text/javascript\">
		<!--
		jQuery(function($) {
			$('#cdaccordion_$acc_id').accordion({
				header: '.header',
				active: $select,
				animated: $animated,
				autoHeight: $autoheight,
				event: '$event',
				collapsible: true
			});
		});
		-->
		</script>";
        $document->addCustomTag($custom_js);
        
        $custom_css = "
		#cdaccordion_$acc_id h3 {
			margin-top: $spacebetween;
		}
		";
        $document->addStyleDeclaration($custom_css);

        unset($custom_js, $custom_css);

        $html = '';
        
    	if ($box) $box = ' style="' . $box . '"';
        
        $html .= '<div class="' . $style . '"' . $box . '>';

        $html .= '<div id="cdaccordion_' . $acc_id . '">'; // open <ul> tag

        switch ($mode)
        {
            case 'id':
            	
                if (explode(':', $set_articles))
                {
                    $set_articles = explode(':', $set_articles);
                } else
                {
                    return;
                }

                foreach ($set_articles as $article_id)
                {
                    $article = $this->renderItem($article_id);
                    
                    $user = & JFactory::getUser();
					$aid = $user->get('aid');
                    
                    if ($article->access <= $aid)
                    {
                        $title = strip_tags($article->title);
                        $fulltext = $article->fulltext;
                        $text = $article->text;

                        if ($fulltext and $readmore)
                        {
                            $text = $text . $article->link;
                        } else
                        {
                            $text = $text . '';
                        }

                        $html .= $this->setInner($title, $text, $process_plugins);
                    }
                    unset($item);
                }

                unset($set_articles);
                break;

            case 'manually':
                
                $array = explode('||||', $set_articles);

                foreach ($array as $article)
                {

                    $array = explode('::', $article);

                    $title = strip_tags((isset($array[0]) ? $array[0] : ''));
                    $text = (isset($array[1]) ? $array[1] : '');
                    $html .= $this->setInner($title, $text, $process_plugins);
                }

                break;
            default:
                return;
                break;
        }

        $html .= '</div>'; // close </ul> tag

        $html .= '</div>';
        
        return $html;

    }
    
/*
	 * Load and render article
	 */
	function renderItem($id = 0)
	{		
		// Create and load the content table row
		$item = & JTable::getInstance('content');
		$item->load($id);
		
		require_once (JPATH_SITE . DS . 'components' . DS . 'com_content' . DS . 'helpers' . DS . 'route.php'); // article routing
		
		$item->text = $item->introtext;
		$item->readmore = (trim($item->fulltext) != '');
		$item->link = JRoute::_(ContentHelperRoute::getArticleRoute($item->id, $item->catid, $item->sectionid));
		
		if (isset($item->link) && $item->readmore)
		{
			$item->link = '<div class="readmore"><a class="readmore" href="' . $item->link .
                '" title="' . JText::_('READ_MORE_LINK') . '">' . JText::_('READ_MORE_LINK') .
                '</a></div>';
		} else
		{
			$item->link = '';
		}
		
		return $item;
	}

    function setInner($title = '', $text = '', $plugins = 0)
    {
        $html = '';
        $html .= '<h3 class="header"><a href="#">' . $title . '</a></h3>';
        $html .= '<div>' . ($plugins ? $item->text = JHTML::_
            ('content.prepare', $text) : $text) . '</div>';
        $html .= '</li>';
        return $html;
    }

}

?>
